@@ -21,7 +21,6 @@
 #include <stdio.h>
 #include <sys/errno.h>
 #include <sys/types.h>
-#include <sys/mman.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <stdint.h>
@@ -39,7 +38,8 @@ struct vector files_vector = {.eltsize = sizeof(struct filename)};
 
 char *name_separator;
 char *dir_separator;
-char *eat_prefix = "./";
+char *default_eat_prefix = "./";
+char *eat_prefix;
 
 int gpicker_bytes_readen;
 int gpicker_load_stdin_too;
@@ -47,6 +47,11 @@ int gpicker_load_stdin_too;
 static volatile
 int reading_aborted;
 
+void init_loading(void)
+{
+	eat_prefix = default_eat_prefix;
+}
+
 static
 void add_filename(char *p, int dirlength)
 {
@@ -141,9 +146,27 @@ void read_filenames_abort(void)
 	reading_aborted = 1;
 }
 
+static
+void read_all(int fd, char *data, int size)
+{
+	int readen;
+	while (size > 0) {
+		readen = read(fd, data, size);
+		if (readen < 0) {
+			if (errno == EINTR)
+				continue;
+			perror("read_all");
+			exit(1);
+		}
+		if (readen == 0)
+			abort();
+		data += readen;
+		size -= readen;
+	}
+}
+
 void read_filenames_from_mlocate_db(int fd)
 {
-#if !defined(__APPLE__) && !defined(__MACH__)
 	static char read_buffer[65536];
 
 	timing_t start;
@@ -157,26 +180,38 @@ void read_filenames_from_mlocate_db(int fd)
 		perror("read_filenames_from_mlocate_db:fstat");
 		exit(1);
 	}
-	data = mmap(0, (size_t)st.st_size, PROT_READ, MAP_SHARED, fd, 0);
-	if (data == MAP_FAILED) {
-		perror("read_filenames_from_mlocate_db:mmap");
-		exit(1);
-	}
 
+	data = xmalloc(st.st_size);
+	read_all(fd, data, st.st_size);
+
+	/* see man 5 mlocate.db */
 	char *prefix = data + 0x10;
-	int prefix_len = strlen(prefix) + 1;
+	int prefix_len = strlen(prefix);
 	char *end = data + st.st_size;
-	char *strings = prefix + prefix_len + ntohl(*((uint32_t *)(data + 0x8))) + 0x10;
+	char *strings = prefix + prefix_len + ntohl(*((uint32_t *)(data + 0x8))) + 0x11;
 
 	if (prefix[prefix_len-2] == '/')
 		prefix_len--;
 
+	if (eat_prefix != default_eat_prefix) {
+		prefix_len = 0;
+		if (eat_prefix[0] != 0) {
+			fprintf(stderr, "mlocate project type supports only default and empty prefixes\n");
+			exit(1);
+		}
+	}
+
 	while (strings < end) {
 		// dir name is in strings
 		int len = strlen(strings);
 		int read_buffer_len = strlen(strings + prefix_len);
-		memcpy(read_buffer, strings + prefix_len, read_buffer_len);
-		if (read_buffer_len)
+		char *used_prefix = strings + prefix_len;
+		if (prefix_len != 0 && *used_prefix == '/') {
+			used_prefix++;
+			read_buffer_len--;
+		}
+		memcpy(read_buffer, used_prefix, read_buffer_len);
+		if (read_buffer_len && read_buffer[read_buffer_len-1] != '/')
 			read_buffer[read_buffer_len++] = '/';
 
 		char *p = strings + len + 1;
@@ -204,10 +239,16 @@ void read_filenames_from_mlocate_db(int fd)
 		strings = p + 0x11;
 	}
 
+	free(data);
+
+	if (gpicker_load_stdin_too) {
+		read_filenames(0);
+	}
+
 	if (!dont_sort) {
 		start = start_timing();
-		_quicksort_top(files, nfiles, sizeof(struct filename), (int (*)(const void *, const void *))filename_compare, files + FILTER_LIMIT);
+		_quicksort_top(files, nfiles, sizeof(struct filename),
+			       (int (*)(const void *, const void *))filename_compare, files + FILTER_LIMIT);
 		finish_timing(start, "initial qsort");
 	}
-#endif
 }
