@@ -468,7 +468,7 @@ void setup_signals(void)
 static
 GOptionEntry entries[] = {
 	{"version", 0, 0, G_OPTION_ARG_NONE, &show_version, "show version", 0},
-	{"project-type", 't', 0, G_OPTION_ARG_STRING, &project_type, "respect ignored files for given kind of VCS (default, git, bzr, hg, guess)", 0},
+	{"project-type", 't', 0, G_OPTION_ARG_STRING, &project_type, "respect ignored files for given kind of VCS (default, git, bzr, hg, guess, mlocate)", 0},
 	{"disable-bzr", 0, 0, G_OPTION_ARG_NONE, &disable_bzr, "disable autodetection of Bazaar project type", 0},
 	{"disable-hg", 0, 0, G_OPTION_ARG_NONE, &disable_hg, "disable autodetection of Mercurial project type", 0},
 	{"name-separator", 'n', 0, G_OPTION_ARG_STRING, &name_separator, "separator of filenames from stdin (\\0 is default)", 0},
@@ -513,7 +513,10 @@ int check_parents(char* name)
 			break;
 		}
 	}
-	fchdir(cwd);
+	if (fchdir(cwd) < 0) {
+		perror("cannot chdir back");
+		exit(1);
+	}
 	return rv;
 }
 
@@ -608,8 +611,9 @@ void parse_options(int argc, char **argv)
 	if (read_stdin) {
 		if (project_type)
 			fprintf(stderr, "Warning: project type with stdin input has no effect\n");
-	} else
+	} else if (!project_type || strcmp(project_type, "mlocate")) {
 		enter_project_dir();
+	}
 }
 
 static
@@ -673,6 +677,8 @@ int simple_main(int, char **);
 
 int main(int argc, char **argv)
 {
+	init_loading();
+
 	char *gpicker = basename(xstrdup(argv[0]));
 	if (!strcmp(gpicker, "gpicker-simple"))
 		return simple_main(argc, argv);
